// *=================================================*
// * NapalmBomber  2003 by Sascha Willems           *
// * E-Mail : webmaster@delphigl.de                  *
// * Page   : http://www.delphigl.de                 *
// *=================================================*
// * This file is part of the "Bomberman"-Tutorial   *
// * series for http://www.delphigl.com              *
// *=================================================*
// * You may use this file in your own projects, but *
// * use it at your own risk ;-)                     *
// *=================================================*
// * global.pas                                      *
// * Last changed : 03/09/2003                       *
// * Changes :                                       *
// *  - Initial Release                              *
// *=================================================*

unit global;

interface

uses
 Classes,
 IniFiles,
 SysUtils,

 OpenGL12,
 glBMP;

const
 // Dateiversion (sollte nur dann gendert werden, wenn etwas am Dateiformat verndert wird)
 FileVersion          = '1.0';
 // Kosntanten fr spezielle Felder
 FSpecialPlayer1Start = 1;
 FSpecialPlayer2Start = 2;
 FSpecialPlayer3Start = 3;
 FSpecialPlayer4Start = 4;

type
 PFieldType = ^TFieldType;
 TFieldType = record
   Texture  : String;
   FType    : Byte;
   FSpecial : Byte;
  end;
 TTextureManager  = class(TObject)
   Texture     : array of TglBMP;
   TextureName : THashedStringList;
   constructor Create; overload;
   destructor Destroy; override;
   procedure AddTexturesInDir(pDirName, pFileMask : String);
   procedure AddTexture(pFileName, pTextureName : String);
   procedure BindTexture(pTextureName : String);
  end;
 TMap = object
   PlayGround          : array of array of TFieldType;
   DefaultFieldTexture : String;
   procedure SetSize(pSize : Integer);
   procedure FillBorder(pTexture : String;pFieldType : Integer);
   procedure FillGround(pTexture : String;pFieldType : Integer);
   procedure Draw;
   procedure SaveToFile(pFileName : String);
   procedure LoadFromFile(pFileName : String);
  end;

var
 TextureManager : TTextureManager;
 SpecialTex     : array of TglBMP;
 SelField       : PFieldType;         // Zeiger auf unter dem Mauscursor liegendes Feld
 SBufferHit     : Integer;            // OpenGL Name des unter dem Cursor liegenden Feldes
 Editor         : Boolean;            // Wenn =TRUE, dann werden spezielle Editoropterationen ausgefhrt
 Map            : TMap;

implementation

// =============================================================================
// =============================================================================
//  TTextureManager
// =============================================================================
// =============================================================================

// =============================================================================
//  TTextureManager.Create                                       - CONSTRUCTOR -
// =============================================================================
constructor TTextureManager.Create;
begin
inherited;
TextureName := THashedStringList.Create;
end;

// =============================================================================
//  TTextureManager.Destroy                                       - DESTRUCTOR -
// =============================================================================
destructor TTextureManager.Destroy;
var
 i : Integer;
begin
if Length(Texture) > 0 then
 for i := Low(Texture) to High(Texture) do
  Texture[i].Free;
TextureName.Free;
inherited;
end;

// =============================================================================
//  TTextureManager.AddTexturesInDir
// =============================================================================
procedure TTextureManager.AddTexturesInDir(pDirName, pFileMask : String);
var
 SR : TSearchRec;
begin
ChDir(pDirName);
if FindFirst(pFileMask, faAnyFile, SR) = 0 then
 repeat
 AddTexture(SR.Name, SR.Name);
 until FindNext(SR) <> 0;
FindClose(SR);
end;


// =============================================================================
//  TTextureManager.AddTexture
// =============================================================================
procedure TTextureManager.AddTexture(pFileName, pTextureName : String);
begin
SetLength(Texture, Length(Texture)+1);
TextureName.Add(pTextureName);
Texture[High(Texture)] := TglBMP.Create(pFileName);
Texture[High(Texture)].GenTexture(False, False);
end;

// =============================================================================
//  TTextureManager.BindTexture
// =============================================================================
procedure TTextureManager.BindTexture(pTextureName : String);
begin
if TextureName.IndexOf(pTextureName) > -1 then
 begin
 glEnable(GL_TEXTURE_2D);
 Texture[TextureName.IndexOf(pTextureName)].Bind;
 end
else
 glDisable(GL_TEXTURE_2D);
end;

// =============================================================================
// =============================================================================
//  TMap
// =============================================================================
// =============================================================================

// =============================================================================
//  TMap.SetSize
// =============================================================================
procedure TMap.SetSize(pSize : Integer);
var
 i : Integer;
begin
SetLength(PlayGround, pSize);
for i := Low(PlayGround) to High(PlayGround) do
 SetLength(PlayGround[i], pSize);
end;

// =============================================================================
//  TMap.FillBorder
// =============================================================================
procedure TMap.FillBorder(pTexture : String;pFieldType : Integer);
var
 i : Integer;
begin
for i := Low(PlayGround) to High(PlayGround) do
 begin
 PlayGround[i,0].Texture                := pTexture;
 PlayGround[i,0].FType                  := pFieldType;
 PlayGround[i,High(PlayGround)].Texture := pTexture;
 PlayGround[i,High(PlayGround)].FType   := pFieldType;
 PlayGround[0,i].Texture                := pTexture;
 PlayGround[0,i].FType                  := pFieldType;
 PlayGround[High(PlayGround),i].Texture := pTexture;
 PlayGround[High(PlayGround),i].FType   := pFieldType;
 end;
end;

// =============================================================================
//  TMap.FillGround
// =============================================================================
procedure TMap.FillGround(pTexture : String;pFieldType : Integer);
var
 i,j : Integer;
begin
for i := Low(PlayGround)+1 to High(PlayGround)-1 do
 for j := Low(PlayGround[i])+1 to High(PlayGround[i])-1 do
  begin
  PlayGround[i,j].Texture := pTexture;
  PlayGround[i,j].FType   := pFieldType;
  end;
end;

// =============================================================================
//  TMap.Draw
// =============================================================================
procedure TMap.Draw;
var
OpenGLID : Integer;
 i,j     : Integer;
begin
OpenGLID := 0;
if Length(Map.PlayGround) > 0 then
 for i := Low(Map.PlayGround) to High(Map.PlayGround) do
  for j := Low(Map.PlayGround[i]) to High(Map.PlayGround[i]) do
   begin
   // Feldkennung auf den Namestack legen
   glLoadName(OpenGLID);
   TextureManager.BindTexture(PlayGround[i,j].Texture);
   if OpenGLID = SBufferHit then
    begin
    SelField := @Map.PlayGround[i,j];
    glDisable(GL_TEXTURE_2D);
    glColor3f(0,1,0);
    end;
    case PlayGround[i,j].FType of
     0   : begin // Bodenflche
           glBegin(GL_QUADS);
            glTexCoord2f(0, 0); glVertex3f(i,   0, j);
            glTexCoord2f(1, 0); glVertex3f(i+1, 0, j);
            glTexCoord2f(1,-1); glVertex3f(i+1, 0, j+1);
            glTexCoord2f(0,-1); glVertex3f(i,   0, j+1);
           glEnd;
           end;
     1,2 : begin // Block
           glBegin(GL_QUADS);
            // Oben
            glTexCoord2f(0,0); glVertex3f(i,   1, j);
            glTexCoord2f(1,0); glVertex3f(i+1, 1, j);
            glTexCoord2f(1,1); glVertex3f(i+1, 1, j+1);
            glTexCoord2f(0,1); glVertex3f(i,   1, j+1);
            // Hinten
            glTexCoord2f(0,0); glVertex3f(i,   0, j+1);
            glTexCoord2f(1,0); glVertex3f(i+1, 0, j+1);
            glTexCoord2f(1,1); glVertex3f(i+1, 1, j+1);
            glTexCoord2f(0,1); glVertex3f(i,   1, j+1);
            // Vorne
            glTexCoord2f(0,0); glVertex3f(i,   0, j);
            glTexCoord2f(1,0); glVertex3f(i+1, 0, j);
            glTexCoord2f(1,1); glVertex3f(i+1, 1, j);
            glTexCoord2f(0,1); glVertex3f(i,   1, j);
            // Links
            glTexCoord2f(0,0); glVertex3f(i,  0, j);
            glTexCoord2f(1,0); glVertex3f(i,  0, j+1);
            glTexCoord2f(1,1); glVertex3f(i,  1, j+1);
            glTexCoord2f(0,1); glVertex3f(i,  1, j);
            // Rechts
            glTexCoord2f(0,0); glVertex3f(i+1,  0, j);
            glTexCoord2f(1,0); glVertex3f(i+1,  0, j+1);
            glTexCoord2f(1,1); glVertex3f(i+1,  1, j+1);
            glTexCoord2f(0,1); glVertex3f(i+1,  1, j);
           glEnd;
           end;
    end;
   glEnable(GL_TEXTURE_2D);
   glColor3f(1,1,1);
   if (PlayGround[i,j].FType = 2) and Editor then
    begin
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    SpecialTex[4].Bind;
    glColor4f(1,1,1,0.3);
    glBegin(GL_QUADS);
     glTexCoord2f(0, 0); glVertex3f(i,   1, j);
     glTexCoord2f(1, 0); glVertex3f(i+1, 1, j);
     glTexCoord2f(1,-1); glVertex3f(i+1, 1, j+1);
     glTexCoord2f(0,-1); glVertex3f(i,   1, j+1);
    glEnd;
    glColor4f(1,1,1,1);
    glDisable(GL_BLEND);
    glDepthFunc(GL_LESS);
    end;
   if Map.PlayGround[i,j].FSpecial > 0 then
    begin
    glDepthFunc(GL_LEQUAL);
    SpecialTex[Map.PlayGround[i,j].FSpecial-1].Bind;
    glBegin(GL_QUADS);
     glTexCoord2f(0, 0); glVertex3f(i,   0, j);
     glTexCoord2f(1, 0); glVertex3f(i+1, 0, j);
     glTexCoord2f(1,-1); glVertex3f(i+1, 0, j+1);
     glTexCoord2f(0,-1); glVertex3f(i,   0, j+1);
    glEnd;
    glDepthFunc(GL_LESS);
    end;
   inc(OpenGLID);
   end;
end;

// =============================================================================
//  TMap.SaveToFile
// =============================================================================
procedure TMap.SaveToFile(pFileName : String);
var
 F   : TextFile;
 i,j : Integer;
begin
AssignFile(F, pFileName);
Rewrite(F);
WriteLn(F, FileVersion);
WriteLn(F, Length(PlayGround));
WriteLn(F, DefaultFieldTexture);
for i := Low(PlayGround) to High(PlayGround) do
 for j := Low(PlayGround) to High(PlayGround) do
  with PlayGround[i,j] do
   begin
   WriteLn(F, Texture);
   WriteLn(F, FType);
   WriteLn(F, FSpecial);
   end;
CloseFile(F);
end;

// =============================================================================
//  TMap.LoadFromFile
// =============================================================================
procedure TMap.LoadFromFile(pFileName : String);
var
 F    : TextFile;
 i,j  : Integer;
 Size : Integer;
 S    : String;
begin
AssignFile(F, pFileName);
Reset(F);
ReadLn(F, S);
// Abbruch, wenn die Datei einer anderen Version vorliegt
if S <> FileVersion then
 begin
 CloseFile(F);
 exit;
 end;
ReadLn(F, Size);
ReadLn(F, DefaultFieldTexture);
Map.SetSize(Size);
for i := Low(PlayGround) to High(PlayGround) do
 for j := Low(PlayGround) to High(PlayGround) do
  with PlayGround[i,j] do
   begin
   ReadLn(F, Texture);
   ReadLn(F, FType);
   ReadLn(F, FSpecial);
   end;
CloseFile(F);
end;


initialization
 TextureManager := TTextureManager.Create;

finalization
 TextureManager.Free;

end.
